source_apps_config()
{
	cd $BUILD_TOOLS_DIR
	if [ -e featureconfig.sh ];then
		. featureconfig.sh
	else
		display_error "Error. Unable to get feature configuration."
	fi
	cd -
}
################# PROJECT SPECIFIC FUNCTIONS #################
#
# Different libraries needed for specific features/applications
# such as c++, ssl etc may be present in the toolchain but by 
# default are not copied to target root filesystem to save space. 
# Only the libraries that were chosen during menuconfig get 
# copied in target root filesystem.
# All such possible libraries should be included here.
#################################################################

select_lib()
{
	echo
}

#
# Depending on the applications selected duing menuconfig
# add the specific directories to the list. The build.sh
# file in each directory that handles the application specific
# build would be invoked for each application.
#
select_opensource_apps()
{
	source_apps_config
#	OPENSOURCE_APP_LIST="busybox-1.00 bridge-utils mtd linux-atm-2.4.1 br2684ctl-20040226-1"
	OPENSOURCE_APP_LIST="busybox-1.00 mtd linux-atm-2.4.1"
	if  [ "$IFX_CONFIG_DNSRELAY" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST dnrd-2.19"
	fi
	if  [ "$IFX_CONFIG_EBTABLES" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST ebtables-v2.0.6"
	fi
	if [  "$IFX_CONFIG_FIREWALL" = "1" -o "$IFX_CONFIG_NAPT" = "1" -o "$IFX_CONFIG_ALGS" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST iptables-1.2.9"
	fi
	if  [ "$IFX_CONFIG_NTPCLIENT" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST ntpclient"
	fi
	#if  [ "$IFX_CONFIG_OPENSSL" = "1" -o "$IFX_CONFIG_SNMPv3" = "1" -o "$IFX_CONFIG_TR69"  = "1" ]; then
	if  [ "$IFX_CONFIG_OPENSSL" = "1" -o "$IFX_CONFIG_SNMPv3" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST openssl-0.9.8a"
	fi
	if  [ "$IFX_CONFIG_SMASH_SNMP" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST smash-2.0.2"
	fi
	if [ "$CONFIG_MATRIX" = "1" -a "$IFX_CONFIG_TR69"  = "1" ];then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST matrixssl-1-8-3-open matrixtunnel"
	fi
	if  [ "$IFX_CONFIG_OPENSWAN" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST openswan-2.4.6"
	fi
	if [ "$IFX_CONFIG_PPPD" = "1" -o "$IFX_CONFIG_PPPOA" = "1" -o "$IFX_CONFIG_PPPOE" = "1" ]; then
                OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST ppp-2.4.2"
	fi
	if  [ "$IFX_CONFIG_PPPOE_RELAY" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST rp-pppoe-3.5"
	fi
	if  [ "$IFX_CONFIG_QOS" = "1" -o "$IFX_CONFIG_POLICY_ROUTE" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST iproute2"
	fi
	#if  [ "$IFX_CONFIG_RFC2684" = "1" ]; then
	#	OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST br2684ctl-20040226-1"
	#fi
	if  [ "$IFX_CONFIG_RIP" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST zebra-0.94"
	fi
	if  [ "$IFX_CONFIG_TR69"  = "1" -a "$CONFIG_MATRIX" != "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST stunnel-4.11"
	fi
	if  [ "$IFX_CONFIG_DSL_MANAGEMENT" = "1" -o \
		"$IFX_CONFIG_SNMP_AGENT" = "1" -o \
		"$IFX_CONFIG_SNMPv1" = "1" -o \
		"$IFX_CONFIG_SNMPv3" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST net-snmp-5.1"
	fi
	if  [ "$IFX_CONFIG_DDNS" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST ez-ipupdate-3.0.11b8"
	fi
	#if [ "$IFX_CONFIG_DHCP_RELAY" = "1" ]; then
		#OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST dhcp-3.0.1rc14"
	#fi
	if  [ "$IFX_CONFIG_WIRELESS" = "1" ]; then
		if [ "$IFX_CONFIG_WIRELESS_ATH" = "1" ]; then
			OPENSOURCE_APP_LIST="wireless_tools.27 $OPENSOURCE_APP_LIST"
#		elif [ "$IFX_CONFIG_WIRELESS_IFX1350" = "1" ]; then
#			OPENSOURCE_APP_LIST="wireless_tools.27 openssl-0.9.8a $OPENSOURCE_APP_LIST"
		fi
	fi
	if [ "$IFX_CONFIG_WIRELESS_IFX1350" = "1" ]; then
		OPENSOURCE_APP_LIST="wireless_tools.27 openssl-0.9.8a $OPENSOURCE_APP_LIST"
	fi
	if  [ "$IFX_CONFIG_SSH_SERVER" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST dropbear-0.43"
	fi
	if  [ "$IFX_CONFIG_FTP_SERVER" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST linux-ftpd-0.17"
	fi
	#if  [ "$IFX_CONFIG_TFTP_SERVER" = "1" ]; then
		#OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST tftp-hpa-0.40"
	#fi
	if [  "$IFX_CONFIG_SYSSTAT" = "1" ]; then
		OPENSOURCE_APP_LIST="$OPENSOURCE_APP_LIST sysstat-6.0.1"
	fi
}
select_ifx_apps()
{
	source_apps_config
	if  [ "$IFX_CONFIG_DSL_CPE_API" = "1" ]; then
		IFX_APP_LIST="dsl_api"
	fi
	if  [ "$IFX_CONFIG_WIRELESS" = "1" ]; then
		if [ "$IFX_CONFIG_WIRELESS_TSC" = "1" ]; then
			IFX_APP_LIST="$IFX_APP_LIST thomson-wlan-rel-2b"
		elif [ "$IFX_CONFIG_WIRELESS_ATH" = "1" ]; then
			IFX_APP_LIST="$IFX_APP_LIST atheros_wireless/802_11/madwifi/"
#		elif [ "$IFX_CONFIG_WIRELESS_IFX1350" = "1" ]; then
#			IFX_APP_LIST="$IFX_APP_LIST AP-DK8.0.0.6_Linux2.4"
		fi
	fi
	if [ "$IFX_CONFIG_WIRELESS_IFX1350" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST AP-DK8.0.0.6_Linux2.4"
	fi
	if  [ "$IFX_CONFIG_OAM" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST oam"
	fi
	IFX_APP_LIST="$IFX_APP_LIST IFXAPIs"
	if  [ "$IFX_CONFIG_DHRYSTONE" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST dhrystone"
	fi
	if  [ "$IFX_CONFIG_TR69"  = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST tr69"
	fi
	IFX_APP_LIST="$IFX_APP_LIST utilities"
	if  [ "$IFX_CONFIG_TR037" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST ilmid"
	fi
	#if  [ "$IFX_CONFIG_MULTIPLE_VCCS" = "1" ]; then
		#echo IFX_CONFIG_MULTIPLE_VCCS
	#fi
	if  [ "$IFX_CONFIG_ADSL_FIRMWARE_LOADER" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST adsl_firmware_loader"
	fi
	if [ "$IFX_CONFIG_TAPIv3" = "1" -o "$IFX_CONFIG_VOIP" = "1" ];then
		IFX_APP_LIST="$IFX_APP_LIST tapi"
	fi
	if [ "$IFX_CONFIG_TAPIv3" = "1" ];then
		IFX_APP_LIST="$IFX_APP_LIST TAPI_firmware tapi_demo-0.9.6.0 "
	fi
	if [ "$IFX_CONFIG_TAPI" = "1" ];then
		IFX_APP_LIST="$IFX_APP_LIST tapidemo fxodemo "
	fi

	if [  "$IFX_CONFIG_ALGS" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST nfappcfg"
	fi
	if  [ "$IFX_CONFIG_VOIP" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST gateway-sip"
	fi
	if  [ "$IFX_CONFIG_WEB_SERVER" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST ifx_httpd"
		if [ "$IFX_CONFIG_CLI" = "1" ]; then
			IFX_APP_LIST="$IFX_APP_LIST ifx_cli"
		fi
	fi
	if  [ "$IFX_CONFIG_VDSL2" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST vdsl2"
	fi
	if  [ "$IFX_VLAN_SUPPORT" = "1" ]; then
		IFX_APP_LIST="$IFX_APP_LIST nfext"
	fi
}

#
# If we need to build rootfs in some peculiar way
#
build_cust_rootfs()
{
	cd "$BUILD_TOOLS_DIR"
	fakeroot "$BUILD_TOOLS_DIR"/.create_rootfs.sh $@
}

build_firmware()
{
	display_info "Building firmware"
	cd "$BUILD_TOOLS_DIR"
	fakeroot "$BUILD_TOOLS_DIR"/.create_firmware_fs.sh $@
}

build_fullimage()
{
	display_info "Building fullimage"
	cd "$BUILD_TOOLS_DIR"
	fakeroot "$BUILD_TOOLS_DIR"/.create_fullimage.sh $@
}

##########################
# General Setup Functions#
##########################
remove_recursive_links()
{
	has_model_changed
	RC=$?
	if [ $RC -eq 1 -o "$RESTORE" = y ];then
		TARGETDIR="$1"
		shift
		for dir in $@
		do
			for elem in `find "$dir" -printf "%P\n"`
			do 
				if [ -L "$TARGETDIR"/"$elem" ]; then
					rm -f "$TARGETDIR"/"$elem"
				fi
			done
		done
	fi

}

setup_recursive_links()
{
	TARGETDIR="$1"
	shift
	for dir in $@
	do
		for elem in `find "$dir" -printf "%P\n"`
		do 
			if [ ! -e "$TARGETDIR"/"$elem" ];then
				setup_link "$dir"/"$elem" "$TARGETDIR"/"$elem"
			fi
		done
	done

}

setup_kernel_links()
{
	if [ -e .config.old ];then
		OLD_BSP_DIR=`grep 'BSP_DIR=' .config.old | sed "s/BSP_DIR=//" | sed "s/\"//g"`
		remove_recursive_links "$KERNEL_SOURCE_DIR" \
			"$TOPDIR"/source/kernel/ifx/os \
			"$OLD_BSP_DIR"/bsp \
			"$OLD_BSP_DIR"/os
	fi
	#setup_link "$PLATFORM_DIR"/config/"$MODEL"/kernel_config \
		#"$CONFIG_DIR"/ifx_kernel_config
        setup_link "$CONFIG_DIR"/kernel_config \
		"$KERNEL_SOURCE_DIR"/arch/mips/defconfig
	setup_recursive_links "$KERNEL_SOURCE_DIR" \
		"$TOPDIR"/source/kernel/ifx/os \
		"$BSP_DIR"/bsp \
		"$BSP_DIR"/os
	if [ "$CONFIG_DNB" = "y" ]; then
		setup_link "$PLATFORM_DIR"/config/"$MODEL"/model.h\
			"$BSP_DIR"/bsp/include/asm-mips/danube/model.h
	elif [ "$CONFIG_AMZ" = "y" ]; then
		setup_link "$PLATFORM_DIR"/config/"$MODEL"/model.h\
			"$BSP_DIR"/bsp/include/asm-mips/amazon/model.h
	elif [ "$CONFIG_AMZ_SE" = "y" ]; then
		setup_link "$PLATFORM_DIR"/config/"$MODEL"/model.h\
			"$BSP_DIR"/bsp/include/asm-mips/amazon_se/model.h
	fi
}

setup_plat_links()
{
	if [ -e .config.old ];then
		OLD_PLATFORM_DIR=`grep 'PLATFORM_DIR=' .config.old | sed "s/PLATFORM_DIR=//" | sed "s/\"//g"`
		remove_recursive_links "$TOPDIR"/source "$OLD_PLATFORM_DIR"/source
		remove_recursive_links "$TOPDIR"/tools "$OLD_PLATFORM_DIR"/tools
	fi
	setup_recursive_links "$TOPDIR"/source "$PLATFORM_DIR"/source
	setup_recursive_links "$TOPDIR"/tools "$PLATFORM_DIR"/tools
	setup_link "$PLATFORM_DIR"/create_CDs "$TOPDIR" 
}

setup_app_links()
{
	setup_link "$TOOLS_DIR"/featureconfig.h \
		"$IFXAPI_INC_DIR"/ifx_config.h
	if [ "$BUILD_VOB" != "y" ];then
		setup_link "$KERNEL_SOURCE_DIR"/include/asm-mips \
			"$IFXAPI_INC_DIR"/asm
	fi
	if [ "$IFX_ADSL_FIRMWARE_IN_ROOTFS" = "1" ];then
		# Pramod - setup link only if dsl_api exists
		if [ -d ""$USER_IFXSOURCE_DIR"/dsl_api/firmware" ]; then
			setup_link "$PLATFORM_DIR"/source/user/ifx/adsl_firmware_loader/firmware \
			"$USER_IFXSOURCE_DIR"/dsl_api/firmware
		fi
	fi
}

setup_dslapi_links()
{
	# copy DSL APIs source codes from version control server to local disk and create a build.sh link
	if [ "$IFX_CONFIG_DSL_CPE_API" =  "1" ]; then
		display_info "DSL APIs is enabled on $PLATFORM_NAME."
		if [ -e ${USER_IFXSOURCE_DIR}dsl_api/ucm__drv_dsl_cpe_api ]; then
			display_info "Generating DSL APIS dsl_driver building environment"
			rm -rf ${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api
			rm -rf ${USER_IFXSOURCE_DIR}dsl_api/dsl_cpe_control
			cp -af ${USER_IFXSOURCE_DIR}dsl_api/ucm__drv_dsl_cpe_api/driver ${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api
			cp -af ${USER_IFXSOURCE_DIR}dsl_api/ucm__drv_dsl_cpe_api/application/dsl_cpe_control ${USER_IFXSOURCE_DIR}dsl_api/dsl_cpe_control
		fi
		ln -sf ${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api/src/include ${KERNEL_IFX_DIR}os/include/ifx/dsl
	fi
	sed -i "s,MOD_INC_USE_COUNT,," "${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api/src/include/drv_dsl_cpe_os_linux.h"
	sed -i "s,MOD_DEC_USE_COUNT,," "${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api/src/include/drv_dsl_cpe_os_linux.h"
	sed -i "s,MODULE_LICENSE.*,," "${USER_IFXSOURCE_DIR}dsl_api/drv_dsl_cpe_api/src/common/drv_dsl_cpe_os_linux.c"
	find ${USER_IFXSOURCE_DIR}dsl_api -name build_danube_mips.sh | xargs sed -i "s,--inc-soap=1,--inc-soap=0,"
	find ${USER_IFXSOURCE_DIR}dsl_api -name build_danube_mips.sh | xargs sed -i "s,--inc-debug=1,--inc-debug=0,"
}

setup_tapi_links()
{
	if [ "$IFX_CONFIG_TAPIv3" = "1" -a  "$PLATFORM_NAME" = "Danube" ]; then
	#if [ "$IFX_CONFIG_TAPIv3" = "1" ];then
	
	display_info "TAPIv3 is enabled "

	if [ -d "$TOPDIR"/platforms/$PLATFORM_NAME/source/user/ifx/tapi ]; then
		sed -i "s,DEFAULT_TAPI_INCL_PATH=.*,DEFAULT_TAPI_INCL_PATH='${TOPDIR}platforms\/$PLATFORM_NAME\/source\/user\/ifx\/tapi\/drv_tapi'," "$TOPDIR"/platforms/$PLATFORM_NAME/source/user/ifx/tapi/ucm__drv_daa/configure.in
		sed -i "s,DEFAULT_DUSLIC_INCL_PATH=.*,DEFAULT_DUSLIC_INCL_PATH='${TOPDIR}platforms\/$PLATFORM_NAME\/source\/user\/ifx\/tapi\/drv_duslic'," "$TOPDIR"/platforms/$PLATFORM_NAME/source/user/ifx/tapi/ucm__drv_daa/configure.in
		sed -i "s,DEFAULT_VINETIC_INCL_PATH=.*,DEFAULT_VINETIC_INCL_PATH='${TOPDIR}platforms\/$PLATFORM_NAME\/source\/user\/ifx\/tapi\/drv_peb3324\/src'," "$TOPDIR"/platforms/$PLATFORM_NAME/source/user/ifx/tapi/ucm__drv_daa/configure.in
	fi

	if [ -e ${USER_IFXSOURCE_DIR}tapi/ucm__drv_tapi ]; then
		display_info "Generating TAPIv3 drv_tapi building environment"
		rm -rf ${USER_IFXSOURCE_DIR}tapi/drv_tapi
		mkdir ${USER_IFXSOURCE_DIR}tapi/drv_tapi
		cp -afx ${USER_IFXSOURCE_DIR}tapi/ucm__drv_tapi/* ${USER_IFXSOURCE_DIR}tapi/drv_tapi

		# creating symbolic links to correct path 
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/common ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/common
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_tapi/src/common ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/common
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_bufferpool ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_bufferpool
			ln -s  ${USER_IFXSOURCE_DIR}tapi/ucm__drv_tapi/src/lib/lib_bufferpool ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_bufferpool
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_fifo ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_fifo
			ln -s  ${USER_IFXSOURCE_DIR}tapi/ucm__drv_tapi/src/lib/lib_fifo ${USER_IFXSOURCE_DIR}tapi/drv_tapi/src/lib/lib_fifo
		fi
	fi
	if [ -e ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc ]; then
		display_info "Generating TAPIv3 drv_vmmc building environment"
		rm -rf ${USER_IFXSOURCE_DIR}tapi/drv_vmmc
		mkdir ${USER_IFXSOURCE_DIR}tapi/drv_vmmc
		cp -ax ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc/*  ${USER_IFXSOURCE_DIR}tapi/drv_vmmc

		# creating symbolic links to correct path
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/common ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/common
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc/src/common ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/common
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bufferpool ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bufferpool
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc/src/lib/lib_bufferpool ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bufferpool
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_fifo ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_fifo
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc/src/lib/lib_fifo ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_fifo
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bbd ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bbd
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_vmmc/src/lib/lib_bbd ${USER_IFXSOURCE_DIR}tapi/drv_vmmc/src/lib/lib_bbd
		fi
	fi
	if [ -e ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic ]; then
		display_info "Generating TAPIv3 drv_duslic building environment"
		rm -rf ${USER_IFXSOURCE_DIR}tapi/drv_duslic
		mkdir ${USER_IFXSOURCE_DIR}tapi/drv_duslic
		cp -ax ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/*  ${USER_IFXSOURCE_DIR}tapi/drv_duslic

		# creating symbolic links to correct path
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/common ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/common
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/common ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/common
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bufferpool ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bufferpool
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/lib/lib_bufferpool ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bufferpool
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_fifo ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_fifo
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/lib/lib_fifo ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_fifo
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bbd ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bbd
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/lib/lib_bbd ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/lib/lib_bbd
		fi
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/mpc ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/mpc
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/mpc ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/mpc
		fi
	fi
	cd ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src
		LC_PLATFORM_NAME=`echo ${PLATFORM_NAME}|tr "[:upper:]" "[:lower:]"`
		ln -s drv_config_user.${LC_PLATFORM_NAME}.h drv_config_user.h
	cd -
	if [ -e ${USER_IFXSOURCE_DIR}tapi/ucm__drv_daa ]; then
		display_info "Generating TAPIv3 drv_daa building environment"
		rm -rf ${USER_IFXSOURCE_DIR}tapi/drv_daa
		mkdir ${USER_IFXSOURCE_DIR}tapi/drv_daa
		cp -ax ${USER_IFXSOURCE_DIR}tapi/ucm__drv_daa/*  ${USER_IFXSOURCE_DIR}tapi/drv_daa

		# creating symbolic links to correct path
		if [ -L ${USER_IFXSOURCE_DIR}tapi/drv_daa/src/common ]; then
			rm ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/common
			ln -s ${USER_IFXSOURCE_DIR}tapi/ucm__drv_duslic/src/common ${USER_IFXSOURCE_DIR}tapi/drv_duslic/src/common
		fi
	fi
 	cd ${USER_IFXSOURCE_DIR}tapi/
	chmod -R 777 drv_*
	cd -

	fi #end of if [ "$IFX_CONFIG_TAPIv3" -a "$PLATFORM_NAME" == "Danube" ]
}

setup_tr69_links()
{
	if [ "$IFX_CONFIG_TR69" = "1" ]; then
		display_info "TR69 Management is enabled on $PLATFORM_NAME"
		if [ "$IFX_CONFIG_TR69_IGD" != "1" -a  "$IFX_CONFIG_TR69_DEVICE" != "1" ]; then
			display_error "Either TR-069 as a Gateway/Router or Device MUST be selected"
		fi
		if [ "$IFX_CONFIG_TR69_IGD" = "1" -a  "$IFX_CONFIG_TR69_DEVICE" = "1" ]; then
			display_error "Both TR-069 as a Gateway/Router or Device CANNOT be selected"
		fi
		if [ "$IFX_CONFIG_TR69_IGD" = "1" -a  "$IFX_CONFIG_TR69_ADSLWAN" != "1" ]; then
			display_error "ADSLWAN Profile MUST be selected"
		fi
		if [ "$IFX_CONFIG_TR69_IGD" = "1" -a  "$IFX_CONFIG_TR69_ATMLOOPBACK" != "1" ]; then
			display_error "ATMLOOPBACK Profile MUST be selected"
		fi
		if [ "$IFX_CONFIG_LMP" = "1" ]; then
			if [ "$IFX_CONFIG_TR64" != "1" -a  "$IFX_CONFIG_UPNP" != "1" ]; then
				display_error "Either LMP as a TR-064 or UPnP MUST be selected"
			fi
			if [ "$IFX_CONFIG_TR64" = "1" -a  "$IFX_CONFIG_UPNP" = "1" ]; then
				display_error "Both LMP as a TR-064 or UPnP CANNOT be selected"
			fi
		fi
		if [ "$IFX_CONFIG_TR69_SSL" != "1" -o "$CONFIG_MATRIX" != "1" ]; then
			display_error "LFP SSL MUST be selected"
		fi
		display_info "Generating TR69 build environment"
		cd "$TR69_SRC_DIR"
		#pwd
		./site_prep.sh
		cd "$BUILD_TOOLS_DIR"
	fi
}

setup_voip_links()
{
	#if [ "$IFX_CONFIG_VOIP" = "1" -a "$PLATFORM_NAME" = "Danube" ]; then
	if [ "$IFX_CONFIG_VOIP" = "1" ]; then
		display_info "VOIP support is enabled on $PLATFORM_NAME"
		display_info "Generating VOIP build environment"
		cd "$VOIP_SRC_DIR"
		#pwd
		./site_prep.sh
		cd "$BUILD_TOOLS_DIR"
	fi
}

plat_setup()
{
	source_apps_config
	setup_plat_links
	setup_kernel_links
	setup_app_links
	setup_tapi_links
	setup_dslapi_links
	apps_setup
	setup_voip_links
	setup_tr69_links
	setup_link "$CONFIG_DIR"/busybox_config \
	"$USER_OPENSOURCE_DIR"/busybox-1.00/ifx_config
	cd "$BUILD_TOOLS_DIR"
	REL_VER=`cat .ifx_package_version`
	# Pramod - update version only if the file is writeable
	if [ -w ""$ROOTFS_SOURCE_DIR"/etc/version" ]; then
		echo "$REL_VER" > "$ROOTFS_SOURCE_DIR"/etc/version
	fi
	if [ -e "$BUILD_ROOTFS_DIR" ];then
		echo "$REL_VER" > "$BUILD_ROOTFS_DIR"/etc/version
	fi
	echo ""$PLATFORM_NAME"-"$REL_VER"-"$MODEL"" > "$KERNEL_SOURCE_DIR"/.hhl_target_lspname

        #cd "$ROOTFS_SOURCE_DIR"
        #rm -f etc/hostname
        #touch etc/hostname
        #chmod 777 etc/hostname
       	#echo "Malta" > etc/hostname
    	#cp -f etc/hostname "$BUILD_ROOTFS_DIR"/etc 2> /dev/null
        cd "$BUILD_TOOLS_DIR"
}
