/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN

#include "drv_dsl_cpe_api.h"
#if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) || \
    defined (INCLUDE_DSL_CPE_API_AMAZON_S)

#include "drv_dsl_cpe_danube.h"

#ifdef __cplusplus
   extern "C" {
#endif

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_DEVICE

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_BitAllocationNSCGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_BitAllocationNsc_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nIndex = 0, nSize = 12, nMsgIdx = 0;
   DSL_uint16_t buf[12];
   DSL_uint16_t nAddr;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_uint16_t nNSC;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_BitAllocationNSCGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not read current line state"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return nErrCode;
   }

   if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Call to this function is possible in "
         "the SHOWTIME state only!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   /* Get number of used tones according to the current ADSL mode*/
   nErrCode = DSL_DRV_DANUBE_UsedTonesNumberGet(pContext, pData->nDirection, &nNSC);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Used tones number get failed"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return nErrCode;
   }

   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_BAT_DS;
   }
   else
   {
      nAddr = DSL_CMV_ADDRESS_INFO_BAT_US;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   for (nIndex = 0; nIndex < nNSC / 2; nIndex += nSize)
   {
      if (nIndex + nSize >= nNSC / 2)
      {
         nSize = nNSC / 2 - nIndex;
      }

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: read BAT: [%d:%d]"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nIndex, nSize));
      
      if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIndex, nSize, buf)) == DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "));
         for (nMsgIdx = 0; nMsgIdx < nSize; nMsgIdx++)
         {
            pData->data.bitAllocationNsc.nNSCData[(nIndex + nMsgIdx) * 2]
               = (DSL_uint8_t) (buf[nMsgIdx] & 0xFF);
            pData->data.bitAllocationNsc.nNSCData[(nIndex + nMsgIdx) * 2 + 1]
               = (DSL_uint8_t) ((buf[nMsgIdx] >> 8) & 0xFF);
            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "%d %d ", pData->data.bitAllocationNsc.nNSCData[(nIndex + nMsgIdx) * 2],
               pData->data.bitAllocationNsc.nNSCData[(nIndex + nMsgIdx) * 2 + 1]));
         }
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "DSL_DRV_CRLF));
      }
      else
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - CMV fail, Group INFO Address %d Index %d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nAddr, nIndex));
      }
   }

   pData->data.bitAllocationNsc.nNumData = nNSC;

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_BitAllocationNSCGet, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_uint32_t DSL_DRV_DEV_G997_GainAllocationNscGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_GainAllocationNsc_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nIndex = 0, nSize = 12, nMsgIdx = 0;
   DSL_uint16_t buf[12];
   DSL_uint16_t nAddr;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_uint16_t nNSC;
   DSL_uint16_t nGainPs = 0;
   DSL_uint16_t ExcMrgRed[2]={0};

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_GainAllocationNscGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not read current line state!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return nErrCode;
   }

   if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Call to this function is possible in "
         "the SHOWTIME state only!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   /* Get number of used tones according to the current ADSL mode*/
   nErrCode = DSL_DRV_DANUBE_UsedTonesNumberGet(pContext, pData->nDirection, &nNSC);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Used tones number get failed"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return nErrCode;
   }

   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_GAIN_DS;
   }
   else
   {
      nAddr = DSL_CMV_ADDRESS_INFO_GAIN_US;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
          DSL_CMV_ADDRESS_INFO_EXCESS_MARGIN, 0, 2, ExcMrgRed)
          != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get Excess Margin Reduction!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         nErrCode = DSL_ERROR;
      }
   }

   for (nIndex = 0; (nIndex < nNSC) && (nErrCode == DSL_SUCCESS); nIndex += nSize)
   {
      if (nIndex + nSize >= nNSC)
      {
         nSize = nNSC - nIndex;
      }

      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: read Gain allocation: [%d:%d]"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nIndex, nSize));
      if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIndex, nSize, buf)) == DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "));
         for (nMsgIdx = 0; nMsgIdx < nSize; nMsgIdx++)
         {
            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "(%hu)", buf[nMsgIdx]));
            if (pData->nDirection == DSL_DOWNSTREAM)
            {
               /* Values in Q8.8 dB*/
               nGainPs = buf[nMsgIdx] == 0x8000 ? 0x0 : buf[nMsgIdx] - ExcMrgRed[0];
            }
            else if (pData->nDirection == DSL_UPSTREAM)
            {
               nGainPs = buf[nMsgIdx] >> 4;
            }

            pData->data.gainAllocationNsc.nNSCData[nIndex + nMsgIdx] = nGainPs;

            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "%hu ", pData->data.gainAllocationNsc.nNSCData[nIndex + nMsgIdx]));
         }
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "DSL_DRV_CRLF));
      }
      else
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - CMV fail, Group INFO Address %d Index %d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nAddr, nIndex));
      }
   }

   pData->data.gainAllocationNsc.nNumData = nNSC;

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_GainAllocationNscGet, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_SnrAllocationNscGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_SnrAllocationNsc_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nIndex = 0, nSize = 12, nMsgIdx = 0;
   DSL_uint16_t buf[12];
   DSL_uint16_t nAddr, nSnrAdd, intPart = 0;
   DSL_uint16_t nMode = 0, nMode1 = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_uint16_t nNSC = 0;
   DSL_uint16_t hdlcCmd[8] = {0}, nHdlcRxLenMax;
   DSL_int_t nHdlcRxLen;
   DSL_uint16_t hdlcRxBuffer[68] = {0};
   DSL_uint16_t nDsRangeD, nDsRangeU;
   DSL_AnnexType_t nAnnex;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_SnrAllocationNSCGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /*$$ ND FIXME: Get the maximum length of Rx buffer and apply it */

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not read current line state!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return nErrCode;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nMode);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive internal Adsl mode "
         " from the device specific data of DSL Context!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1, nMode1);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive internal Adsl mode 1 "
         " from the device specific data of DSL Context!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nAnnexType, nAnnex);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive Annex type"
         " from the DSL Context!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Call to this function is possible in "
         "the SHOWTIME state only!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   /* read HDLC max rx buffer length in case of UpStream and ADSL2/2+ */
   if (pData->nDirection == DSL_UPSTREAM && (nMode >= 0x100 || nMode1 != 0))
   {
      if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_ME_HDLC_PARAMS, 1,
         1, &nHdlcRxLenMax)) != DSL_SUCCESS)
      {
         /* unlock mutex and return Error */
         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
         return nErrCode;
      }
   }

   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      /* Get number of used tones according to the current ADSL mode*/
      nErrCode = DSL_DRV_DANUBE_UsedTonesNumberGet(pContext, pData->nDirection, &nNSC);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Used tones number get failed"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
      else
      {
         nAddr = DSL_CMV_ADDRESS_INFO_SNRA_DS;

         for (nIndex = 0; nIndex < nNSC; nIndex += nSize)
         {
            if (nIndex + nSize >= nNSC)
            {
               nSize = nNSC - nIndex;
            }

            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "DSL[%02d]: read Snr allocation table: [%d:%d]"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nIndex, nSize));

            if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
               nAddr, nIndex, nSize, buf)) == DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "));
               for (nMsgIdx = 0; nMsgIdx < nSize; nMsgIdx++)
               {
                  intPart = ((buf[nMsgIdx] >> 8) + 32) * 2;
                  pData->data.snrAllocationNsc.nNSCData[nIndex + nMsgIdx] =
                     (DSL_uint8_t)((buf[nMsgIdx] & 0xFF) < 128 ? intPart : intPart + 1);

                  DSL_DEBUG(DSL_DBG_MSG, (pContext,
                     "(%d) %d ", buf[nMsgIdx],
                     pData->data.snrAllocationNsc.nNSCData[nIndex + nMsgIdx]));
               }
               DSL_DEBUG(DSL_DBG_MSG, (pContext, "   "DSL_DRV_CRLF));
            }
            else
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext,
                  "DSL[%02d]: ERROR - CMV fail, Group INFO Address %d Index %d!"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nAddr, nIndex));
            }
         }
      }
   }
   else if (pData->nDirection == DSL_UPSTREAM)
   {
      if (nMode >= 0x100 || nMode1 != 0)
      {
         /* Adsl2/2+ */
         /* Determine number of values within result depend on Annex */
         if ((nAnnex == DSL_ANNEX_A) || (nAnnex == DSL_ANNEX_I)
            || (nAnnex == DSL_ANNEX_L))
         {
            nNSC = 32;
         }
         else if (nAnnex == DSL_ANNEX_B)
         {
            nNSC = 64;
         }
         else if (nAnnex == DSL_ANNEX_M || nAnnex == DSL_ANNEX_J)
         {
            /* $$ND: FIXME: According to US-Mask number of
               returned values is 64 max but its range is 32..64 */
            nNSC = 64;
         }
      }
      else
      {
         /* Adsl1 */
         nNSC = 0;
         memset(&pData->data.snrAllocationNsc.nNSCData, 0, DSL_MAX_NSC);
      }

      if (nNSC != 0)
      {
         /* 64 + 4 - size of allocated on stack rx buffer */
         nSize = (nHdlcRxLenMax > 64) ? 64 : nHdlcRxLenMax;
         for (nIndex = 0; nIndex < nNSC; nIndex += nSize)
         {
            if (nIndex + nSize >= nNSC)
            {
               nSize = nNSC - nIndex;
            }

            if (nMode >= 0x4000 || nMode1 != 0)
            {
               hdlcCmd[0] = 0x181;
               hdlcCmd[1] = 0x4 | ((nIndex & 0xFF) << 8);
               hdlcCmd[2] = (((nIndex + nSize - 1) & 0xFF) << 8)
                  | ((nIndex & 0xFF00) >> 8);
               hdlcCmd[3] = (((nIndex + nSize - 1) & 0xFF00) >> 8);
            }
            else
            {
               hdlcCmd[0] = 0x181;
               hdlcCmd[1] = 0x4 | ((nIndex & 0xFF) << 8);
               hdlcCmd[2] = ((nIndex + nSize - 1) & 0xFF);
            }

            if ((nErrCode = DSL_DRV_DANUBE_HdlcWrite(pContext, DSL_FALSE,
               (DSL_uint8_t *)hdlcCmd,
               (nMode >= 0x4000 || nMode1 != 0) ? 8 : 6)) == DSL_SUCCESS)
            {
               DSL_WAIT (1);

               nErrCode = DSL_DRV_DANUBE_HdlcRead(pContext, DSL_FALSE,
                  (DSL_uint8_t*)hdlcRxBuffer, nSize + 4, &nHdlcRxLen);
               if (nErrCode == DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: HDLC READ ADSL2%s %d"
                     DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
                     (nMode >= 0x4000 || nMode1 != 0) ? "+" : "", nHdlcRxLen));
                  for (nMsgIdx = 0; nMsgIdx < nSize / 2; nMsgIdx++)
                  {
                     pData->data.snrAllocationNsc.nNSCData[2 *
                        (nMsgIdx + nIndex) + 1] =
                        (hdlcRxBuffer[nMsgIdx + 2] >> 8) & 0xFF;
                     pData->data.snrAllocationNsc.nNSCData[2 *
                        (nMsgIdx + nIndex)] =
                        (hdlcRxBuffer[nMsgIdx + 2]) & 0xFF;
                  }
                  DSL_DEBUG(DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));
               }
            }
         }
      }
      else
      {
         if (nMode >= 0x100 || nMode1 != 0)
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: Unsupported Annex type was "
               "reported by firmware, could not continue"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            nErrCode = DSL_ERR_NOT_SUPPORTED;
         }
      }
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Direction was not selected!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERROR;
   }

   if (nErrCode == DSL_SUCCESS)
   {
      pData->data.snrAllocationNsc.nNumData = nNSC;

      /* convert values for DS */
      if (pData->nDirection == DSL_DOWNSTREAM)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_DS_BIT_LOAD_FIRST, 0, 1, &nDsRangeD);
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
               DSL_CMV_ADDRESS_INFO_DS_BIT_LOAD_LAST, 0, 1, &nDsRangeU);
            if (nErrCode == DSL_SUCCESS)
            {
               if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
                  DSL_CMV_ADDRESS_INFO_EXCESS_MARGIN, 0, 1, &nSnrAdd))
                  != DSL_SUCCESS)
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Couldn't get EXCESSMGN for %s!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
                  nErrCode = DSL_ERROR;
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_MSG, (pContext, " nSnrAdd=%h "DSL_DRV_CRLF,
                     (DSL_int16_t)nSnrAdd));
                  for (nIndex = 0; nIndex < nDsRangeD; nIndex++)
                  {
                     if (pData->data.snrAllocationNsc.nNSCData[nIndex] == 0x40)
                     {
                        pData->data.snrAllocationNsc.nNSCData[nIndex] = 0xFF;
                     }
                  }
                  for (nIndex = nDsRangeD; nIndex <= nDsRangeU; nIndex++)
                  {
                     pData->data.snrAllocationNsc.nNSCData[nIndex] += (DSL_int16_t)nSnrAdd;
                  }
                  for (nIndex = nDsRangeU + 1; nIndex < nNSC; nIndex++)
                  {
                     if (pData->data.snrAllocationNsc.nNSCData[nIndex] == 0x40)
                     {
                        pData->data.snrAllocationNsc.nNSCData[nIndex] = 0xFF;
                     }
                  }
               }
            }
         }
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_SnrAllocationNSCGet, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   This function does not check pContext pointer, modem availability and
   pData pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_G997_ActPSD_Get(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN DSL_AccessDir_t nDirection,
   DSL_OUT DSL_int16_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nData = 0;
   DSL_uint16_t nNomPsdAbs = 0;
   DSL_uint16_t nPcb = 0;
   DSL_int16_t nRmsgi = 0, nRmsgiSub = 0;
   DSL_int_t nIndexNomPsdAbs = 0,
             nIndexPcb = 0, nIndexRmsgi = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_ActPSD_Get"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (nDirection == DSL_DOWNSTREAM)
   {
      nIndexPcb = DSL_CMV_PMD_INDEX_PCB_DS;
      nIndexRmsgi = DSL_CMV_PMD_INDEX_RMSGI_DS;
      nIndexNomPsdAbs = DSL_CMV_PMD_INDEX_NOMINAL_PSD_DS;
   }
   else if (nDirection == DSL_UPSTREAM)
   {
      nIndexPcb = DSL_CMV_PMD_INDEX_PCB_US;
      nIndexRmsgi = DSL_CMV_PMD_INDEX_RMSGI_US;
      nIndexNomPsdAbs = DSL_CMV_PMD_INDEX_NOMINAL_PSD_US;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Direction was not selected!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive line state from the "
         "DSL Context!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   if (nLineState == DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE ||
       nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      *pData = 0;
      return DSL_SUCCESS;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, DSL_CMV_ADDRESS_INFO_PMD,
      nIndexPcb, 1, &nData) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't get PCB for %s!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[nDirection + 1]));
      nErrCode = DSL_ERROR;
   }
   else
   {
      nPcb = nData;

      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_PMD, nIndexNomPsdAbs,
         1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get NOMPSDAbs for %s!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[nDirection + 1]));
         nErrCode = DSL_ERROR;
      }
      else
      {
         nNomPsdAbs = nData;

         if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, DSL_CMV_ADDRESS_INFO_PMD,
            nIndexRmsgi, 1, &nData) != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't get RMSGI for %s!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[nDirection + 1]));
            nErrCode = DSL_ERROR;
         }
         else
         {
            nRmsgi = (DSL_int16_t) nData;

            if (nDirection == DSL_UPSTREAM)
            {
               if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
                  DSL_CMV_ADDRESS_INFO_EXCESS_MARGIN, 0, 1, &nData)
                  != DSL_SUCCESS)
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Couldn't get EXCESSMGN for %s!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[nDirection + 1]));
                  nErrCode = DSL_ERROR;
               }
               else
               {
                  nRmsgiSub = (DSL_int16_t) nData;
               }
            }

            if (nErrCode == DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nPcb=%hu "
                  "nRmsgi=%hd nRmsgiSub(excessMgn)=%hd nNomPsdAbs=%hu",
                  DSL_DEV_NUM(pContext), nPcb, nRmsgi, nRmsgiSub, nNomPsdAbs));

               *pData = (((DSL_int16_t)(nNomPsdAbs - 65536)) * 256 - nPcb * 10 * 256
                  + (nRmsgi - nRmsgiSub) * 10) / 256;
            }
         }
      }
   }

   if (nErrCode != DSL_SUCCESS)
   {
      *pData = -901;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_ActPSD_Get,retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}


/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_LineStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_LineStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   /* internal mode values for Danube */
   DSL_uint16_t nMode = 0, nMode1 = 0;
   DSL_boolean_t bDiagComplete = DSL_FALSE;
   DSL_uint16_t nAddr, nData;
   DSL_uint16_t hdlcCmd[2];
   DSL_uint16_t hdlcRxBuffer[32];
   DSL_int_t nHdlcRxLen = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_LineStatusGet(%s)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   if (pData->nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      return DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if(pData->nDeltDataType != DSL_DELT_DATA_SHOWTIME)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERROR;
   }

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the line is in SHOWTIME state.*/
   if ( (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
        (nCurrentState != DSL_LINESTATE_SHOWTIME_NO_SYNC) )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Function is only available in the SHOWTIME!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }
   
   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_LINE_STATUS_DS;
   }
   else if (pData->nDirection == DSL_UPSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_LINE_STATUS_US;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Direction was not selected!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, bLoopDiagnosticsCompleted, bDiagComplete);

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nMode);

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1, nMode1);

   memset(&pData->data, 0, sizeof(DSL_G997_LineStatus_t));
   
   /* Reset Line Status structure according to "special values"*/
   pData->data.LATN   = -32768;
   pData->data.SATN   = -32768;
   pData->data.SNR    = -32768;
   /* No special value*/
   pData->data.ATTNDR = 0;
   pData->data.ACTPS  = -901;
   pData->data.ACTATP = -512;

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   /* LATN value will be read using CMV for
      ADSL1   : upstream and downstream
      ADSL2/2+: downstream only */
   if ((nMode <= 8 && nMode1 == 0) || pData->nDirection == DSL_DOWNSTREAM)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr,
         1, 1, (DSL_uint16_t*)&pData->data.LATN) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get LATN for %s!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
         nErrCode = DSL_ERROR;
      }
   }
   /* In case of ADSL2/2+ upstream value for LATN will be requested via OHC
      channel to get a value that is always up-to-date */
   else
   {
      hdlcCmd[0] = 0x0181;
      hdlcCmd[1] = 0x21;

      if (DSL_DRV_DANUBE_HdlcWrite (pContext, DSL_FALSE, (char *) hdlcCmd, 4)
         == DSL_SUCCESS)
      {
         DSL_WAIT (1);

         if (DSL_DRV_DANUBE_HdlcRead (pContext, DSL_FALSE,
                (char *)&hdlcRxBuffer, 32 * 2, &nHdlcRxLen) == DSL_SUCCESS)
         {
            if (nHdlcRxLen <= 0)
            {
               DSL_DEBUG( DSL_DBG_WRN,
                  (pContext, "DSL[%02d]: WARNING - HDLC read invalid RxLen!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
               nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
            }
            else
            {
               pData->data.LATN = DSL_Le2Cpu(hdlcRxBuffer[1]);
            }
         }
         else
         {
            DSL_DEBUG( DSL_DBG_WRN,
               (pContext, "DSL[%02d]: WARNING - HDLC Read failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
                  
            nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
         }
      }
      else
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: WARNING - HDLC Send failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: LATN: ",DSL_DEV_NUM(pContext)));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, "(%hd / 10dB) ", pData->data.LATN));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, ""DSL_DRV_CRLF));

   if (nErrCode >= DSL_SUCCESS)
   {
      /* SATN value will be read using CMV for
         ADSL1   : upstream and downstream
         ADSL2/2+: downstream only */
      if ((nMode <= 8 && nMode1 == 0) || pData->nDirection == DSL_DOWNSTREAM)
      {
         if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr,
            2, 1, (DSL_uint16_t*)&pData->data.SATN) != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't get SATN for %s!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
            nErrCode = DSL_ERROR;
         }
      }
      /* In case of ADSL2/2+ upstream value for SATN will be requested via OHC
         channel to get a value that is always up-to-date */
      else
      {
         hdlcCmd[0] = 0x0181;
         hdlcCmd[1] = 0x22;

         if (DSL_DRV_DANUBE_HdlcWrite (pContext, DSL_FALSE, (char *) hdlcCmd, 4)
            == DSL_SUCCESS)
         {
            DSL_WAIT (1);

            if (DSL_DRV_DANUBE_HdlcRead (pContext, DSL_FALSE,
                   (char *) &hdlcRxBuffer, 32 * 2, &nHdlcRxLen) == DSL_SUCCESS)
            {
               if (nHdlcRxLen <= 0)
               {
                  DSL_DEBUG( DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: WARNING - HDLC read invalid RxLen!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                           
                  nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
               }
               else
               {
                  pData->data.SATN = DSL_Le2Cpu(hdlcRxBuffer[1]);
               }
            }
            else
            {
               DSL_DEBUG( DSL_DBG_WRN,
                  (pContext, "DSL[%02d]: WARNING - HDLC Read failed!"DSL_DRV_CRLF,
                   DSL_DEV_NUM(pContext)));
                        
               nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
            }
         }
         else
         {
            DSL_DEBUG( DSL_DBG_WRN,
               (pContext, "DSL[%02d]: WARNING - HDLC Send failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
                     
            nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: SATN: ", DSL_DEV_NUM(pContext)));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, "(%hd / 10dB) ", pData->data.SATN));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, ""DSL_DRV_CRLF));

   if (nErrCode >= DSL_SUCCESS)
   {
      /* SNR value will be read using CMV for ADSL modes in downstream */
      if (pData->nDirection == DSL_DOWNSTREAM)
      {
         if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            DSL_CMV_ADDRESS_PLAM_SNRM_0_1DB,
            0, 1, (DSL_uint16_t*)&pData->data.SNR) != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't get SNR for %s!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
            nErrCode = DSL_ERROR;
         }
      }
      else
      {
         /* Upstream SNR value for ADSL1 will be read using CMV that reports
            the training SNR value (this is NOT changing during showtime!) */
         if ((nMode <= 8 && nMode1 == 0) || pData->nDirection == DSL_DOWNSTREAM)
         {
            if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr,
               3, 1, (DSL_uint16_t*)&pData->data.SNR) != DSL_SUCCESS)
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Couldn't get SNR for %s!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
               nErrCode = DSL_ERROR;
            }
         }
         /* Upstream SNR value for ADSL2/2+ will be read via OHC channel to get
            a value that is always up-to-date */
         else
         {
            hdlcCmd[0] = 0x0181;
            hdlcCmd[1] = 0x23;

            if (DSL_DRV_DANUBE_HdlcWrite (pContext, DSL_FALSE, (char *)hdlcCmd, 4) == DSL_SUCCESS)
            {
               DSL_WAIT (1);

               if (DSL_DRV_DANUBE_HdlcRead (pContext, DSL_FALSE, 
                      (char *) &hdlcRxBuffer, 32 * 2, &nHdlcRxLen) == DSL_SUCCESS)
               {
                  if (nHdlcRxLen <= 0)
                  {
                     DSL_DEBUG( DSL_DBG_WRN,
                        (pContext, "DSL[%02d]: WARNING - HDLC read invalid RxLen!"DSL_DRV_CRLF,
                        DSL_DEV_NUM(pContext)));
                           
                     nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
                  }
                  else
                  {
                     pData->data.SNR = DSL_Le2Cpu(hdlcRxBuffer[1]);
                  }
               }
               else
               {
                  DSL_DEBUG( DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: WARNING - HDLC Read failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                        
                  nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
               }
            }
            else
            {
               DSL_DEBUG( DSL_DBG_WRN,
                  (pContext, "DSL[%02d]: WARNING - HDLC Send failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
                     
               nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
            }
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: SNR: ", DSL_DEV_NUM(pContext)));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, "(%hd / 10dB) ", pData->data.SNR));
   DSL_DEBUG(DSL_DBG_MSG, (pContext, ""DSL_DRV_CRLF));

   /* Get ATTNDR*/
   if (nErrCode >= DSL_SUCCESS)
   {
      if ((nMode <= 8 && nMode1 == 0) || pData->nDirection == DSL_DOWNSTREAM)
      {
         if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr,
               4, 2, (DSL_uint16_t*)&pData->data.ATTNDR) != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't get ATTNDR for %s!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
            nErrCode = DSL_ERROR;
         }
         else
         {
            pData->data.ATTNDR = ((pData->data.ATTNDR & 0xFFFF) << 16) |
               (pData->data.ATTNDR & 0xFFFF0000) >> 16;
         }
      }
      /* Upstream ATTNDR value for ADSL2/2+ will be read via OHC channel to get
         a value that is always up-to-date */
      else
      {
         hdlcCmd[0] = 0x0181;
         hdlcCmd[1] = 0x24;

         if (DSL_DRV_DANUBE_HdlcWrite (pContext, DSL_FALSE,
                (char *) hdlcCmd, 4) == DSL_SUCCESS)
         {
            DSL_WAIT (1);

            if (DSL_DRV_DANUBE_HdlcRead (pContext, DSL_FALSE, 
                   (char *) &hdlcRxBuffer, 32 * 2, &nHdlcRxLen) == DSL_SUCCESS)
            {
               if (nHdlcRxLen <= 0)
               {
                  DSL_DEBUG( DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: WARNING - HDLC read invalid RxLen!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                        
                  nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
               }
               else
               {
                  pData->data.ATTNDR = (((DSL_uint32_t)DSL_Le2Cpu(hdlcRxBuffer[1])) << 16 )|
                                        ((DSL_uint32_t)DSL_Le2Cpu(hdlcRxBuffer[2]));
               }
            }
            else
            {
               DSL_DEBUG( DSL_DBG_WRN,
                  (pContext, "DSL[%02d]: WARNING - HDLC Read failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
                    
               nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
            }
         }
         else
         {
            DSL_DEBUG( DSL_DBG_WRN,
               (pContext, "DSL[%02d]: WARNING - HDLC Send failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
                  
            nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: (ATTNDR: %08x)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), pData->data.ATTNDR));

   /* Get ACTATP*/
   if (nErrCode >= DSL_SUCCESS)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr,
         6, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get ACTATP for %s!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
            
         nErrCode = DSL_ERROR;
      }
      else
      {
         pData->data.ACTATP = nData;      
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: (ACTATP: %d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), pData->data.ACTATP));

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   /* Get ACTPS*/
   if (nErrCode >= DSL_SUCCESS)
   {
      if (DSL_DRV_DANUBE_G997_ActPSD_Get(pContext, pData->nDirection, &(pData->data.ACTPS))
          != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get ACTPS for %s!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_DBG_PRN_DIR[pData->nDirection + 1]));
         nErrCode = DSL_ERROR;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: (ACTPS: %d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), pData->data.ACTPS));   

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_LineStatusGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_LineInventoryGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_LineInventory_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_XTUDir_t nDirection = pData->nDirection;
   DSL_G997_LineInventoryData_t *pLineInventory;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_LineInventoryGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DRV_MUTEX_LOCK(pContext->dataMutex);

   pLineInventory = nDirection == DSL_NEAR_END ? &(pContext->lineInventoryNe) :
                                                 &(pContext->lineInventoryFe);

   memcpy(&(pData->data), pLineInventory, sizeof(DSL_G997_LineInventoryData_t));

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_LineInventoryGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_LineInventorySet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_LineInventoryNe_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_LineInventorySet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DRV_MUTEX_LOCK(pContext->dataMutex);

   /* Set VersionNumber*/
   memcpy(
      pContext->lineInventoryNe.VersionNumber,
      pData->data.VersionNumber,
      DSL_G997_LI_MAXLEN_VERSION);

   /* Set SystemVendorID*/
   memcpy(
      pContext->lineInventoryNe.SystemVendorID,
      pData->data.SystemVendorID,
      DSL_G997_LI_MAXLEN_VENDOR_ID);

   /* Set SerialNumber*/
   memcpy(
      pContext->lineInventoryNe.SerialNumber,
      pData->data.SerialNumber,
      DSL_G997_LI_MAXLEN_SERIAL);

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_LineInventorySet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_LineTransmissionStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_LineTransmissionStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_LineTransmissionStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the specified line is in SHOWTIME state.*/
   if ( nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC ||
      nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC )
   {
      pData->data.nLineTransmissionStatus = DSL_G997_LINE_TRANSMISSION_AVAILABLE;
   }
   else
   {
      pData->data.nLineTransmissionStatus = DSL_G997_LINE_TRANSMISSION_NOT_AVAILABLE;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_LineTransmissionStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_ChannelStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_ChannelStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_G997_ChannelStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_CHANNEL_RANGE(pData->nChannel);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the line is in SHOWTIME state.*/
   if ( (nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ||
        (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC) )
   {
      nErrCode = DSL_DRV_DANUBE_ChannelStatusGet(
                    pContext, pData->nChannel,
                    pData->nDirection, &pData->data);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Function is only available in the SHOWTIME!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_G997_ChannelStatusGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_LastStateTransmittedGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_G997_LastStateTransmitted_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_uint16_t nAddr = 0;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_G997_LastStateTransmittedGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   if (pData->nDirection == DSL_DOWNSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS_DS;
   }
   else if (pData->nDirection == DSL_UPSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS_US;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Direction was not selected!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddr, 1, 1,
      &pData->data.nLastStateTransmitted);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_G997_LastStateTransmittedGet, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_FramingParameterStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN const DSL_uint8_t nChannel,
   DSL_OUT DSL_G997_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t   nAddrNFEC = 0, nAddrRFEC, nAddrLSYMB, nAddrINTLVDEPTH;
   DSL_boolean_t bNFECAddrWr = DSL_FALSE;
   DSL_LatencyPath_t nLp;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_FramingParameterStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the line is in SHOWTIME state.*/
   if ( (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
        (nCurrentState != DSL_LINESTATE_SHOWTIME_NO_SYNC) )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Function is only available in the SHOWTIME!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   /* Update channel status for the case if it was not apdated yet */
   nErrCode = DSL_DRV_DANUBE_ChannelStatusUpdate(pContext);

   if (nErrCode == DSL_SUCCESS)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nLPath[nChannel], nLp);
      pData->nLPATH = (nLp == DSL_LATENCY_IP_LP0 ? 0 :
         nLp == DSL_LATENCY_FP_LP1 ? 1 : 0);
   }

   if (nDirection == DSL_UPSTREAM)
   {
      if (nLp == DSL_LATENCY_IP_LP0)
      {
         nAddrNFEC = DSL_CMV_ADDRESS_RATE_DATA_NFEC_US_LP0;
      }
      else if (nLp == DSL_LATENCY_FP_LP1)
      {
         nAddrNFEC = DSL_CMV_ADDRESS_RATE_DATA_NFEC_US_LP1;
      }
      else
      {
         bNFECAddrWr = DSL_TRUE;
      }
      nAddrRFEC = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_RP;
      nAddrLSYMB = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_LP;
      nAddrINTLVDEPTH = DSL_CMV_ADDRESS_CNFG_DATA_INTLVDEPTH_US;
   }
   else
   {
      if (nLp == DSL_LATENCY_IP_LP0)
      {
         nAddrNFEC = DSL_CMV_ADDRESS_RATE_DATA_NFEC_DS_LP0;
      }
      else if (nLp == DSL_LATENCY_FP_LP1)
      {
         nAddrNFEC = DSL_CMV_ADDRESS_RATE_DATA_NFEC_DS_LP1;
      }
      else
      {
         bNFECAddrWr = DSL_TRUE;
      }
      nAddrRFEC = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_RP;
      nAddrLSYMB = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_LP;
      nAddrINTLVDEPTH = DSL_CMV_ADDRESS_CNFG_DATA_INTLVDEPTH_DS;
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if (bNFECAddrWr == DSL_FALSE)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
            nAddrNFEC, nChannel, 1, &pData->nNFEC);
      }
      else
      {
         pData->nNFEC = 0;
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrRFEC, nChannel, 1, &pData->nRFEC);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrLSYMB, nChannel, 1, &pData->nLSYMB);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrINTLVDEPTH, nChannel, 1, &pData->nINTLVDEPTH);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      /* This parameter is for VDSL only */
      pData->nINTLVBLOCK = 0;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_FramingParameterStatusGet, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_DeltHlinScaleGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN DSL_DeltDataType_t nDeltDataType,
   DSL_OUT DSL_G997_DeltHlinScaleData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_DeltHlinScaleGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (nDeltDataType == DSL_DELT_DATA_SHOWTIME)
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if (nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (pContext->DELT != DSL_NULL)
      {
         memcpy(pData,
            (nDirection == DSL_UPSTREAM) ? &pContext->DELT->hlinScaleDataUs :
            &pContext->DELT->hlinScaleDataDs,
            (nDirection == DSL_UPSTREAM) ?
            sizeof(DSL_G997_DeltHlinInternalUsData_t) :
            sizeof(DSL_G997_DeltHlinInternalDsData_t));
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_DeltHlinScaleGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_DeltHlinGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN DSL_DeltDataType_t nDeltDataType,
   DSL_OUT DSL_G997_DeltHlinData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_DeltHlinGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (nDeltDataType == DSL_DELT_DATA_SHOWTIME)
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if (nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (pContext->DELT != DSL_NULL)
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: DSL_DEV_G997_DeltHlinGet: return data"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         memcpy(pData,
            ((nDirection == DSL_UPSTREAM) ?
            (DSL_void_t*)&pContext->DELT->hlinDataUs :
            (DSL_void_t*)&pContext->DELT->hlinDataDs),
            (nDirection == DSL_UPSTREAM) ?
            sizeof(DSL_G997_DeltHlinInternalUsData_t)
            : sizeof(DSL_G997_DeltHlinInternalDsData_t));
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_DeltHlinGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_DeltHlogGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN DSL_DeltDataType_t nDeltDataType,
   DSL_OUT DSL_G997_DeltHlogData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nMsgIdx, i;
   DSL_uint16_t *pIntData, nNumData, nMeasurementTime;
   DSL_uint8_t nGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_DeltHlogGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (nDeltDataType == DSL_DELT_DATA_SHOWTIME)
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if (nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (pContext->DELT != DSL_NULL)
      {
         if (nDirection == DSL_DOWNSTREAM)
         {
            pIntData = pContext->DELT->hlogDataDs.deltHlog.nSCGData;
            nGroupSize = pContext->DELT->hlogDataDs.nGroupSize;
            nNumData = pContext->DELT->hlogDataDs.deltHlog.nNumData;
            nMeasurementTime = pContext->DELT->hlogDataDs.nMeasurementTime;
         }
         else
         {
            pIntData = pContext->DELT->hlogDataUs.deltHlog.nSCGData;
            nGroupSize = pContext->DELT->hlogDataUs.nGroupSize;
            nNumData = pContext->DELT->hlogDataUs.deltHlog.nNumData;
            nMeasurementTime = pContext->DELT->hlogDataUs.nMeasurementTime;
         }

         /* if actual group size != 1, values should be spread */
         if (nGroupSize != 1)
         {
            /* check the sizes */
            if (nNumData * nGroupSize > DSL_MAX_NSC)
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Storage for data too small!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
            }
            else
            {
               pData->deltHlog.nNSCData[0] = pIntData[0];
               for (nMsgIdx = 1; nMsgIdx < DSL_MAX_NSC; nMsgIdx += nGroupSize)
               {
                  for (i = 0; i < nGroupSize; i++)
                  {
                     pData->deltHlog.nNSCData[nMsgIdx + i + 1] =
                        pIntData[(nMsgIdx-1)/2];
                  }
               }

               /* final fixing */
               pData->deltHlog.nNumData = nNumData * nGroupSize;
               pData->nGroupSize = 1;
               pData->nMeasurementTime = nMeasurementTime;
            }
         }
         else
         {
            if (nDirection == DSL_DOWNSTREAM)
            {
               memcpy(pData, &pContext->DELT->hlogDataDs,
                  sizeof(DSL_G997_DeltHlogInternalDsData_t));
            }
            else
            {
               memcpy(pData, &pContext->DELT->hlogDataUs,
                  sizeof(DSL_G997_DeltHlogInternalUsData_t));
            }
         }
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_DeltHlogGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DEV_G997_DeltQLNGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN DSL_DeltDataType_t nDeltDataType,
   DSL_OUT DSL_G997_DeltQlnData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nNumData, nMeasurementTime;
   DSL_int_t nMsgIdx, i;
   DSL_uint8_t *pIntData, nGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_DeltQLNGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (nDeltDataType == DSL_DELT_DATA_SHOWTIME)
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if (nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (pContext->DELT != DSL_NULL)
      {
         if (nDirection == DSL_DOWNSTREAM)
         {
            pIntData = pContext->DELT->qlnDataDs.deltQln.nSCGData;
            nGroupSize = pContext->DELT->qlnDataDs.nGroupSize;
            nNumData = pContext->DELT->qlnDataDs.deltQln.nNumData;
            nMeasurementTime = pContext->DELT->qlnDataDs.nMeasurementTime;
         }
         else
         {
            pIntData = pContext->DELT->qlnDataUs.deltQln.nSCGData;
            nGroupSize = pContext->DELT->qlnDataUs.nGroupSize;
            nNumData = pContext->DELT->qlnDataUs.deltQln.nNumData;
            nMeasurementTime = pContext->DELT->qlnDataUs.nMeasurementTime;
         }

         /* if actual group size != 1, values should be spread */
         if (nGroupSize != 1)
         {
            /* check the sizes */
            if (nNumData * nGroupSize > DSL_MAX_NSC)
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Storage for data too small!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
            }
            else
            {
               /* 0th tone*/
               pData->deltQln.nNSCData[0] = pIntData[0];
               for (nMsgIdx = 1; nMsgIdx < DSL_MAX_NSC; nMsgIdx += nGroupSize)
               {
                  for (i = 0; i < nGroupSize; i++)
                  {
                     pData->deltQln.nNSCData[nMsgIdx + i] =
                        pIntData[(nMsgIdx-1)/2];
                  }
               }

               /* final fixing */
               pData->deltQln.nNumData = nNumData * nGroupSize;
               pData->nGroupSize = 1;
               pData->nMeasurementTime = nMeasurementTime;
            }
         }
         else
         {
            if (nDirection == DSL_DOWNSTREAM)
            {
               memcpy(pData, &pContext->DELT->qlnDataDs,
                  sizeof(DSL_G997_DeltQlnInternalDsData_t));
            }
            else
            {
               memcpy(pData, &pContext->DELT->qlnDataUs,
                  sizeof(DSL_G997_DeltQlnInternalUsData_t));
            }
         }
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_DeltQLNGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_Error_t DSL_DRV_DEV_G997_DeltSNRGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN DSL_DeltDataType_t nDeltDataType,
   DSL_OUT DSL_G997_DeltSnrData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_DeltSNRGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (nDeltDataType == DSL_DELT_DATA_SHOWTIME)
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else if (nDeltDataType == DSL_DELT_DATA_DIAGNOSTICS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (pContext->DELT != DSL_NULL)
      {
         if (nDirection == DSL_UPSTREAM)
         {
            DSL_DEBUG( DSL_DBG_MSG,
               (pContext, "DSL[%02d]: Snr data info - nMeasurementTime = %d,"
               "nNumData = %d (%p, %p)"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext),
               pContext->DELT->snrDataUs.nMeasurementTime,
               pContext->DELT->snrDataUs.deltSnr.nNumData,
               &pContext->DELT->snrDataUs.nMeasurementTime,
               &pContext->DELT->snrDataUs.deltSnr.nNumData));
         }
         else
         {
            DSL_DEBUG( DSL_DBG_MSG,
               (pContext, "DSL[%02d]: Snr data info - nMeasurementTime = %d,"
               "nNumData = %d (%p, %p)"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext),
               pContext->DELT->snrDataDs.nMeasurementTime,
               pContext->DELT->snrDataDs.deltSnr.nNumData,
               &pContext->DELT->snrDataDs.nMeasurementTime,
               &pContext->DELT->snrDataDs.deltSnr.nNumData));
         }

         memcpy(pData,
            (nDirection == DSL_UPSTREAM) ?
            (DSL_void_t*)&pContext->DELT->snrDataUs :
            (DSL_void_t*)&pContext->DELT->snrDataDs,
            (nDirection == DSL_UPSTREAM) ?
            sizeof(DSL_G997_DeltSnrInternalUsData_t) :
            sizeof(DSL_G997_DeltSnrInternalDsData_t));
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown DELT data type!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_DeltSNRGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the equivalent ioctl
   \ref DSL_FIO_G997_POWER_MANAGEMENT_STATE_FORCED_TRIGGER
*/
DSL_Error_t DSL_DRV_DEV_G997_PowerManagementStateForcedTrigger(
   DSL_IN DSL_Context_t *pContext,
   DSL_OUT DSL_G997_PowerManagementStateForcedTriggerData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_PowerManagementStatusData_t nPMStatus;
   DSL_int_t nAttempt = 0, i;
   DSL_uint16_t nVal;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_PowerManagementStateForcedTrigger"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get Actual Power Management Status from the DSL CPE context*/
   DSL_CTX_READ(pContext, nErrCode, powerMgmtStatus, nPMStatus);

   switch(pData->nPowerManagementState)
   {
   case DSL_G997_PMSF_L3_TO_L0:
      if(nPMStatus.nPowerManagementStatus == DSL_G997_PMS_L3)
      {
         /* Reset Flag*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bPowerManagementL3Forced, DSL_FALSE);
      }
      else
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext,
            "DSL[%02d]: PMSF L3-L0 requested but not feasible (actual mode: L%d)." DSL_DRV_CRLF ,
            DSL_DEV_NUM(pContext),
            (DSL_int_t)nPMStatus.nPowerManagementStatus));
         nErrCode = DSL_ERROR;
      }
      break;
   case DSL_G997_PMSF_L0_TO_L2:
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext,
         "DSL[%02d]: PMSF L0-L2 requested but not supported." DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_DEVICE;
      break;
   case DSL_G997_PMSF_LX_TO_L3:
      if (nPMStatus.nPowerManagementStatus != DSL_G997_PMS_L3)
      {
         if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            return DSL_ERROR;
         }

         /* retry it DSL_LX_TO_L3_ATTEMPT_COUNT times */
         for (nAttempt = 0; nAttempt < DSL_LX_TO_L3_ATTEMPT_COUNT; nAttempt++)
         {
            /* read previous data in the L3 request CMV */
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNTL,
               DSL_CMV_ADDRESS_CNTL_L3_REQUEST, 0, 1, &nVal);
            if (nErrCode == DSL_SUCCESS)
            {
               nVal |= 1;
               nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_CNTL,
                  DSL_CMV_ADDRESS_CNTL_L3_REQUEST, 0, 1, &nVal);

               DSL_DEBUG(DSL_DBG_MSG,
                  (pContext, "DSL[%02d]: DEBUG - "
                  "DSL_DEV_G997_PowerManagementStateForcedTrigger: "
                  "CMV DSL_CMV_ADDRESS_CNTL_L3_REQUEST,0,1 write %hu"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), nVal));

               nErrCode = DSL_ERROR;

               for (i = 0; i < DSL_LX_TO_L3_TIMEOUT*10; i++)
               {
                  /*$$ ND: sleep for 100 msec, is it enough? */
                  DSL_WAIT(100);
                  /* read L3 request status */
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
                     DSL_CMV_ADDRESS_STAT_L3_REQUEST, 0, 1, &nVal);
                  DSL_DEBUG(DSL_DBG_MSG,
                     (pContext, "DSL[%02d]: DEBUG - "
                     "DSL_DEV_G997_PowerManagementStateForcedTrigger: "
                     "CMV DSL_CMV_ADDRESS_STAT_L3_REQUEST,0,1 read %hu"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), nVal));

                  if (nErrCode != DSL_SUCCESS)
                  {
                     nErrCode = DSL_ERR_MSG_EXCHANGE;
                     break;
                  }
                  if ((nVal & 0x3) == 0)
                  {
                     continue;
                  }
                  else if ((nVal & 0x3) == 1)
                  {
                     nErrCode = DSL_ERR_L3_REJECTED;
                  }
                  else if ((nVal & 0x3) == 2)
                  {
                     nErrCode = DSL_SUCCESS;
                     break;
                  }
                  else
                  {
                     /* read L3 request failure reason */
                     nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
                        DSL_CMV_ADDRESS_STAT_L3_FAILURE_REASON, 0, 1, &nVal);
                     DSL_DEBUG(DSL_DBG_MSG,
                        (pContext, "DSL[%02d]: DEBUG - "
                        "DSL_DEV_G997_PowerManagementStateForcedTrigger: "
                        "CMV DSL_CMV_ADDRESS_STAT_L3_FAILURE_REASON,0,1 read %hu"DSL_DRV_CRLF,
                        DSL_DEV_NUM(pContext), nVal));

                     if (nErrCode != DSL_SUCCESS)
                     {
                        nErrCode = DSL_ERR_MSG_EXCHANGE;
                        break;
                     }
                     if (((nVal >> 4) & 0x15) == 0x5)
                     {
                        nErrCode = DSL_ERR_L3_NOT_IN_L0;
                     }
                     else if (((nVal >> 4) & 0x15) == 0x9)
                     {
                        nErrCode = DSL_ERR_L3_TIMED_OUT;
                     }
                     else
                     {
                        nErrCode = DSL_ERR_L3_UNKNOWN_FAILURE;
                     }
                  }
               }

               if (nErrCode != DSL_ERROR)
               {
                  break;
               }
            }
         }

         if (nErrCode == DSL_ERROR)
         {
            nErrCode = DSL_ERR_L3_TIMED_OUT;
         }

         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

         if (nErrCode == DSL_SUCCESS)
         {
            /*Set flag*/
            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "DSL[%02d]: DEBUG - "
               "DSL_DEV_G997_PowerManagementStateForcedTrigger: "
               "Set L3 power management status"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nVal));
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bPowerManagementL3Forced, DSL_TRUE);

            /* Trigger restart sequence*/
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bAutobootRestart, DSL_TRUE);
         }
      }
      else
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext,
            "DSL[%02d]: PMSF Lx-L3 requested but not feasible (actual mode: L%d)." DSL_DRV_CRLF ,
            DSL_DEV_NUM(pContext),
            (DSL_int_t)nPMStatus.nPowerManagementStatus));
         nErrCode = DSL_ERROR;
      }
      break;
   default:
      break;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_PowerManagementStateForcedTrigger, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
static DSL_uint32_t DSL_DRV_DANUBE_G997_DeltHlinScaleGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;
   DSL_uint16_t nAddr;
   DSL_uint16_t *pData;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_DeltHlinScaleGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (pContext->bLoopDiagnosticsCompleted != DSL_TRUE)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - call is only possible "
         "after diagnostic is complete"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      nErrCode |= DSL_ERROR;
   }
   DSL_CHECK_ERR_CODE();

   if (nDirection == DSL_DOWNSTREAM)
   {
      nAddr = DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS_DS;

      pData = &pContext->DELT->hlinScaleDataDs.nDeltHlinScale;
   }
   else
   {
      nAddr = DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS_US;

      pData = &pContext->DELT->hlinScaleDataUs.nDeltHlinScale;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      nAddr, DSL_CMV_DELT_PREV_STATS_INDEX_HLIN_SC, 1, &nVal);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV "
         "DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS_US(DS) read fail!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      *pData = nVal;
      DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: ERROR - CMV "
         "DSL_CMV_ADDRESS_INFO_DELT_PREV_STATS, nDirection=%d, "
         "Value=%d!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection, nVal));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_DeltHlinScaleGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
static DSL_uint32_t DSL_DRV_DANUBE_G997_DeltHlinGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nIdx = 0, nSize = 6, nMsgIdx;
   DSL_uint16_t buf[12];
   DSL_uint16_t nSizeMax, nAddr;
   DSL_uint16_t nMode = 0, nMode1 = 0;

   /* pointers data should be saved to */
   DSL_G997_ComplexNumber_t *pData;
   DSL_uint16_t *pnMeasurementTime, *pnNumData;
   DSL_uint8_t *pnGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_DeltHlinGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (pContext->bLoopDiagnosticsCompleted != DSL_TRUE)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - "
         "call is only possible after diagnostic is complete!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode |= DSL_ERROR;
   }
   DSL_CHECK_ERR_CODE();

   nMode = pContext->pDevCtx->data.nAdslMode;
   nMode1 = pContext->pDevCtx->data.nAdslMode1;

   if (nDirection == DSL_DOWNSTREAM)
   {
      pData = pContext->DELT->hlinDataDs.deltHlin.nSCGComplexData;
      pnMeasurementTime = &pContext->DELT->hlinDataDs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->hlinDataDs.nGroupSize;
      pnNumData = &pContext->DELT->hlinDataDs.deltHlin.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_HLIN_PS_DS;

      if (nMode >= 0x4000 || nMode1 > 0)
      {
         nSizeMax = 512;
      }
      else
      {
         nSizeMax = 256;
      }
   }
   else
   {
      pData = pContext->DELT->hlinDataUs.deltHlin.nSCGComplexData;
      pnMeasurementTime = &pContext->DELT->hlinDataUs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->hlinDataUs.nGroupSize;
      pnNumData = &pContext->DELT->hlinDataUs.deltHlin.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_HLIN_PS_US;

      nSizeMax = 64;
   }

   for (nIdx = 0; nIdx < nSizeMax; nIdx += nSize)
   {
      if (nIdx + nSize >= nSizeMax)
      {
         nSize = nSizeMax - nIdx;
      }

      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIdx * 2, nSize * 2, buf);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV "
            "DSL_CMV_ADDRESS_INFO_DELT_HLIN_PS_DS(US) read fail, nIdx=%d,"
            " nSize=%d!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nIdx * 2, nSize * 2));
      }
      else
      {
         memcpy (&pData[nIdx],
            &buf, nSize * 2 * sizeof(DSL_uint16_t));
      }
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: CMV "
      "DSL_CMV_ADDRESS_INFO_DELT_HLIN_PS, nDirection=%d,"
      " nSize=%d!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection, nSizeMax));
   for (nMsgIdx = 0; nMsgIdx < nSizeMax; nMsgIdx++)
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext, " %02X%02X",
         pData[nMsgIdx].nReal, 
         pData[nMsgIdx].nImag ));
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));

   *pnNumData = nSizeMax;
   *pnMeasurementTime = 256;
   *pnGroupSize = 1;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_DeltHlinGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
static DSL_uint32_t DSL_DRV_DANUBE_G997_DeltHlogGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nIdx = 0, nSize = 12, nMsgIdx;
   DSL_uint16_t buf[12];
   DSL_uint16_t nSizeMax, nAddr, nAddrMT;
   DSL_uint16_t nMode = 0, nMode1 = 0;
   DSL_AnnexType_t nAnnex;

   /* pointers data should be saved to */
   DSL_uint16_t *pData;
   DSL_uint16_t *pnMeasurementTime, *pnNumData;
   DSL_uint8_t *pnGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_DeltHlogGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (pContext->bLoopDiagnosticsCompleted != DSL_TRUE)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - "
         "call is only possible after diagnostic is complete!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode |= DSL_ERROR;
   }
   DSL_CHECK_ERR_CODE();

   nMode = pContext->pDevCtx->data.nAdslMode;
   nMode1 = pContext->pDevCtx->data.nAdslMode1;
   nAnnex = pContext->nAnnexType;

   if (nDirection == DSL_DOWNSTREAM)
   {
      pData = pContext->DELT->hlogDataDs.deltHlog.nSCGData;
      pnMeasurementTime = &pContext->DELT->hlogDataDs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->hlogDataDs.nGroupSize;
      pnNumData = &pContext->DELT->hlogDataDs.deltHlog.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_HLOG_PS_DS;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_DS;
      nSizeMax = 256;
   }
   else
   {
      pData = pContext->DELT->hlogDataUs.deltHlog.nSCGData;
      pnMeasurementTime = &pContext->DELT->hlogDataUs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->hlogDataUs.nGroupSize;
      pnNumData = &pContext->DELT->hlogDataUs.deltHlog.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_HLOG_PS_US;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_US;

      if ((nAnnex == DSL_ANNEX_A) || (nAnnex == DSL_ANNEX_I)
         || (nAnnex == DSL_ANNEX_L))
      {
         nSizeMax = 32;
      }
      else if (nAnnex == DSL_ANNEX_B)
      {
         nSizeMax = 64;
      }
      else if (nAnnex == DSL_ANNEX_M || nAnnex == DSL_ANNEX_J)
      {
         /* $$ND: FIXME: According to US-Mask number of
            returned values is 64 max but its range is 32..64 */
         nSizeMax = 64;
      }
      else
      {
         nSizeMax = 64;
      }
   }

   for (nIdx = 0; nIdx < nSizeMax; nIdx += nSize)
   {
      if (nIdx + nSize >= nSizeMax)
      {
         nSize = nSizeMax - nIdx;
      }

      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIdx, nSize, buf);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV "
            "DSL_CMV_ADDRESS_INFO_DELT_HLOG_PS_DS(US) read fail, nIdx=%d,"
            " nSize=%d!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nIdx, nSize));
      }
      else
      {
         memcpy (&pData[nIdx], &buf, nSize * sizeof(DSL_uint16_t));
      }
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: CMV "
      "DSL_CMV_ADDRESS_INFO_DELT_HLOG_PS, nDirection=%d, "
      "nSize=%d!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection, nSizeMax));
   for (nMsgIdx = 0; nMsgIdx < nSizeMax; nMsgIdx++)
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext, " %02X", pData[nMsgIdx]));
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      nAddrMT, DSL_CMV_DELT_INDEX_MT_HLOG, 1, buf);
   *pnMeasurementTime = buf[0];

   *pnNumData = nSizeMax;

   /* set real group size, it will be fixed in the API functions */
   if ((nMode >= 0x4000 || nMode1 > 0) && nDirection == DSL_DOWNSTREAM)
   {
      /* in case of ADSL 2+ the values will be doubled */
      *pnGroupSize = 2;
   }
   else
   {
      *pnGroupSize = 1;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_DeltHlogGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DANUBE_G997_DeltQLNGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nIdx = 0, nSize = 12, nMsgIdx = 0;
   DSL_uint16_t buf[12];
   DSL_uint16_t nSizeMax, nAddr, nAddrMT, nMode = 0, nMode1 = 0;
   DSL_AnnexType_t nAnnex;

   /* pointers data should be saved to */
   DSL_uint8_t *pData;
   DSL_uint16_t *pnMeasurementTime, *pnNumData;
   DSL_uint8_t *pnGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_DeltQLNGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (pContext->bLoopDiagnosticsCompleted != DSL_TRUE)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - "
         "call is only possible after diagnostic is complete!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode |= DSL_ERROR;
   }
   DSL_CHECK_ERR_CODE();

   nMode = pContext->pDevCtx->data.nAdslMode;
   nMode1 = pContext->pDevCtx->data.nAdslMode1;
   nAnnex = pContext->nAnnexType;

   if (nDirection == DSL_DOWNSTREAM)
   {
      pData = pContext->DELT->qlnDataDs.deltQln.nSCGData;
      pnMeasurementTime = &pContext->DELT->qlnDataDs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->qlnDataDs.nGroupSize;
      pnNumData = &pContext->DELT->qlnDataDs.deltQln.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_QLN_PS_DS;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_DS;
      nSizeMax = 128;
   }
   else
   {
      pData = pContext->DELT->qlnDataUs.deltQln.nSCGData;
      pnMeasurementTime = &pContext->DELT->qlnDataUs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->qlnDataUs.nGroupSize;
      pnNumData = &pContext->DELT->qlnDataUs.deltQln.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_QLN_PS_US;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_US;

      if ((nAnnex == DSL_ANNEX_A) || (nAnnex == DSL_ANNEX_I)
         || (nAnnex == DSL_ANNEX_L))
      {
         nSizeMax = 16;
      }
      else if (nAnnex == DSL_ANNEX_B)
      {
         nSizeMax = 32;
      }
      else if (nAnnex == DSL_ANNEX_M || nAnnex == DSL_ANNEX_J)
      {
         /* $$ND: FIXME: According to US-Mask number of
            returned values is 64 max but its range is 32..64 */
         nSizeMax = 32;
      }
      else
      {
         nSizeMax = 32;
      }
   }

   for (nIdx = 0; nIdx < nSizeMax; nIdx += nSize)
   {
      if (nIdx + nSize >= nSizeMax)
      {
         nSize = nSizeMax - nIdx;
      }

      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIdx, nSize, buf);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV "
            "DSL_CMV_ADDRESS_INFO_DELT_QLN_PS_DS(US) read fail, nIdx=%d,"
            " nSize=%d!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nIdx, nSize));
      }
      else
      {
         for (nMsgIdx = 0; nMsgIdx < nSize; nMsgIdx++)
         {
            pData[(nIdx + nMsgIdx) * 2] = buf[nMsgIdx] & 0xFF;
            pData[(nIdx + nMsgIdx) * 2 + 1] =
               (buf[nMsgIdx] >> 8) & 0xFF;
         }
      }
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: CMV "
      "DSL_CMV_ADDRESS_INFO_DELT_QLN_PS, nDirection=%d, "
      "nSize=%d!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection, nSizeMax));
   for (nMsgIdx = 0; nMsgIdx < nSizeMax; nMsgIdx++)
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext, " %02X %02X",
         pData[nMsgIdx * 2],
         pData[nMsgIdx * 2 + 1]));
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      nAddrMT, DSL_CMV_DELT_INDEX_MT_QLN, 1, buf);
   *pnMeasurementTime = buf[0];

   *pnNumData = nSizeMax * 2;

   /* set real group size, it will be fixed in the API functions */
   if ((nMode >= 0x4000 || nMode1 > 0) && nDirection == DSL_DOWNSTREAM)
   {
      /* in case of ADSL 2+ the values will be doubled */
      *pnGroupSize = 2;
   }
   else
   {
      *pnGroupSize = 1;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_DeltQLNGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
static DSL_Error_t DSL_DRV_DANUBE_G997_DeltSNRGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nIdx = 0, nSize = 12, nMsgIdx;
   DSL_uint16_t buf[12];
   DSL_uint16_t nSizeMax, nAddr, nAddrMT;
   DSL_uint16_t nDsRangeD, nDsRangeU;
   DSL_uint16_t nMode = 0, nMode1 = 0, intPart = 0;
   DSL_AnnexType_t nAnnex;

   /* pointers data should be saved to */
   DSL_uint8_t *pData;
   DSL_uint16_t *pnMeasurementTime, *pnNumData;
   DSL_uint8_t *pnGroupSize;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_G997_DeltSNRGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   if (pContext->bLoopDiagnosticsCompleted != DSL_TRUE)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - "
         "call is only possible after diagnostic is complete!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return DSL_ERROR;
   }

   nMode = pContext->pDevCtx->data.nAdslMode;
   nMode1 = pContext->pDevCtx->data.nAdslMode1;
   nAnnex = pContext->nAnnexType;

   if (nDirection == DSL_DOWNSTREAM)
   {
      pData = pContext->DELT->snrDataDs.deltSnr.nSCGData;
      pnMeasurementTime = &pContext->DELT->snrDataDs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->snrDataDs.nGroupSize;
      pnNumData = &pContext->DELT->snrDataDs.deltSnr.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_SNR_PS_DS;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_DS;
      if (nMode >= 0x4000 || nMode1 > 0)
      {
         nSizeMax = 512;
      }
      else
      {
         nSizeMax = 256;
      }
   }
   else
   {
      pData = pContext->DELT->snrDataUs.deltSnr.nSCGData;
      pnMeasurementTime = &pContext->DELT->snrDataUs.nMeasurementTime;
      pnGroupSize = &pContext->DELT->snrDataUs.nGroupSize;
      pnNumData = &pContext->DELT->snrDataUs.deltSnr.nNumData;

      nAddr = DSL_CMV_ADDRESS_INFO_DELT_SNR_PS_US;
      nAddrMT = DSL_CMV_ADDRESS_INFO_DELT_MT_US;

      if ((nAnnex == DSL_ANNEX_A) || (nAnnex == DSL_ANNEX_I)
         || (nAnnex == DSL_ANNEX_L))
      {
         nSizeMax = 16;
      }
      else if (nAnnex == DSL_ANNEX_B)
      {
         nSizeMax = 32;
      }
      else if (nAnnex == DSL_ANNEX_M || nAnnex == DSL_ANNEX_J)
      {
         /* $$ND: FIXME: According to US-Mask number of
            returned values is 64 max but its range is 32..64 */
         nSizeMax = 32;
      }
      else
      {
         nSizeMax = 32;
      }
   }

   for (nIdx = 0; nIdx < nSizeMax; nIdx += nSize)
   {
      if (nIdx + nSize >= nSizeMax)
      {
         nSize = nSizeMax - nIdx;
      }

      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         nAddr, nIdx, nSize, buf);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV "
            "DSL_CMV_ADDRESS_INFO_DELT_SNR_PS_DS(US) read fail, nIdx=%d,"
            " nSize=%d!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nIdx, nSize));
      }
      else
      {
         for (nMsgIdx = 0; nMsgIdx < nSize; nMsgIdx++)
         {
            if (nDirection == DSL_DOWNSTREAM)
            {
               intPart = ((buf[nMsgIdx] >> 8) + 32) * 2;
               pData[nIdx + nMsgIdx] =
                  (DSL_uint8_t)((buf[nMsgIdx] & 0xFF) < 128 ? intPart : intPart + 1);
            }
            else
            {
               pData[(nIdx + nMsgIdx) * 2]     = buf[nMsgIdx] & 0xFF;;
               pData[(nIdx + nMsgIdx) * 2 + 1] = (buf[nMsgIdx] >> 8) & 0xFF;
            }
         }
      }
   }

   if (nDirection == DSL_UPSTREAM)
   {
      nSizeMax *= 2;
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: ERROR - CMV "
      "DSL_CMV_ADDRESS_INFO_DELT_SNR_PS, nDirection=%d, "
      "nSize=%d!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection, nSizeMax));
   for (nMsgIdx = 0; nMsgIdx < nSizeMax; nMsgIdx++)
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext, " %02X",
         pData[nMsgIdx]));
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));

   /* convert values for DS */
   if (nDirection == DSL_DOWNSTREAM)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_DS_BIT_LOAD_FIRST, 0, 1, &nDsRangeD);
      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_DS_BIT_LOAD_LAST, 0, 1, &nDsRangeU);
         if (nErrCode == DSL_SUCCESS)
         {
            for (nIdx = 0; nIdx < nDsRangeD; nIdx++)
            {
               if (pData[nIdx] == 0x40)
               {
                  pData[nIdx] = 0xFF;
               }
            }
            for (nIdx = nDsRangeU + 1; nIdx < nSizeMax; nIdx++)
            {
               if (pData[nIdx] == 0x40)
               {
                  pData[nIdx] = 0xFF;
               }
            }
         }
      }
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      nAddrMT, DSL_CMV_DELT_INDEX_MT_SNR, 1, buf);
   *pnMeasurementTime = buf[0];

   *pnNumData = nSizeMax;
   *pnGroupSize = 1;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Snr internal data info - nMeasurementTime = %d,"
      "nNumData = %d (%p, %p)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      *pnMeasurementTime,
      *pnNumData,
      pnMeasurementTime,
      pnNumData));

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_DeltSNRGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_G997_LoopDiagnosticCompleted(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_AccessDir_t nDirection;
   DSL_uint8_t nAutoCount = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_G997_LoopDiagnosticCompleted"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_CTX_POINTER(pContext);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLoopAutoCount, nAutoCount);
   if( nAutoCount > 0 )
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLoopAutoCount, nAutoCount - 1);
   }

/* Do not include additional code */
#ifndef DSL_CPE_STATIC_DELT_DATA
   if (pContext->DELT == DSL_NULL)
   {
      pContext->DELT = DSL_DRV_Malloc(sizeof(DSL_G997_DeltData_t));
   }
#endif

   if (pContext->DELT == DSL_NULL)
   {
      nErrCode = DSL_ERR_MEMORY;
   }
   else
   {
      memset(pContext->DELT, 0, sizeof(DSL_G997_DeltData_t));
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      if (DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      for (nDirection = DSL_UPSTREAM; nDirection <= DSL_DOWNSTREAM; nDirection++)
      {
         nErrCode = DSL_DRV_DANUBE_G997_DeltHlinGet(pContext, nDirection);
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_G997_DeltHlinScaleGet(pContext, nDirection);
            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_G997_DeltHlogGet(pContext, nDirection);
               if (nErrCode == DSL_SUCCESS)
               {
                  nErrCode =  DSL_DRV_DANUBE_G997_DeltQLNGet(pContext, nDirection);
                  if (nErrCode == DSL_SUCCESS)
                  {
                     nErrCode =  DSL_DRV_DANUBE_G997_DeltSNRGet(pContext, nDirection);
                  }
               }
            }
         }
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_G997_LoopDiagnosticCompleted, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#ifdef __cplusplus
}
#endif
#endif /* (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) 
                                       || defined(INCLUDE_DSL_CPE_API_AMAZON_S)*/
