#!/bin/bash

NEW_VERSION=$1
TOP_DIR=.
FILES="configure.in build_win32/*.h"

usage()
{
	echo
	echo "Usage:"
	echo "$0 new_version"
	echo Example: $0 3.4.1
	exit 1
}

checkout ()
{
   local filename=$1
   # 1. Check is the file exists
   if ! test -f ${filename} ; then
      Error "file ${filename} does not exist!"
   fi
   # 2. If the file is not write protected, return
   if test -w ${filename} ; then
      return
   fi
   #3. If file is write protected, check if is in
   #   a ClearCase view
   if cleartool ls -short ${filename} >/dev/null; then
      # Checkout it
      if ! cleartool checkout -nc ${filename} >/dev/null; then
         Error "Cannot checkout ${filename}"
      fi
   else
      # If the file is not in ClearCase, try to
      # make it writable.
      chmod +w ${filename}
      if ! test -w ${filename} ; then
         Error "Cannot make ${filename} writable"
      fi
   fi
}

change_version()
{
   echo checkout ${1}
   checkout ${1}
   echo "${1}" - "${2} - ${NEW_VERSION}"
   sed -i "s/${2}/${NEW_VERSION}/" ${TOP_DIR}/${ff}
}

if [ "$1" == "" ]; then
    echo "Provide a new version"
    usage
    exit 1
else
   NEW_VERSION=${1}
fi

for ff in ${FILES}; do
   OLD_VERSION=`grep "!define VERSION " ${TOP_DIR}/${ff} | sed 's/!define VERSION //'`
   if [ "${OLD_VERSION}" != "" ]; then
      change_version ${ff} ${OLD_VERSION}
   else 
      OLD_VERSION=`grep "PROJECT_NUMBER         = " ${TOP_DIR}/${ff} | sed 's/PROJECT_NUMBER         = //'`
      if [ "${OLD_VERSION}" != "" ]; then
         change_version ${ff} ${OLD_VERSION}
      else
         OLD_VERSION=`grep AC_INIT $ff | cut -d"," -f 2 | sed 's/\[//g' | sed 's/\]//g'`
         if [ "${OLD_VERSION}" != "" ]; then
            change_version ${ff} ${OLD_VERSION}
         else
            OLD_VERSION=`grep "#define VERSION " $ff | sed 's/#define VERSION //g' | sed 's/\"//g'`
            if [ "${OLD_VERSION}" != "" ]; then
               change_version ${ff} ${OLD_VERSION}
            fi
         fi
      fi
   fi
done
